import datetime
import json

from typing import List, Optional
from urllib.parse import urljoin

from pydantic import BaseModel, Field, HttpUrl

from .client import SDKClient, SDKResponse


class SearchClientRequest(BaseModel):
    fname: Optional[str]
    lname: Optional[str]
    mname: Optional[str]
    phone: Optional[str]
    birth: Optional[datetime.date]
    discountnumber: Optional[int]


class Test(BaseModel):
    lname: str
    uuid: str
    fname: str


class Client(BaseModel):
    id: str = Field(alias="_id")
    source: Test = Field(alias="_source")


class ElkStructure(BaseModel):
    hits: List[Client]


class SearchClientResponse(BaseModel):
    hits: ElkStructure


class ElkService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        assert url.endswith("/")
        self._client = client
        self._url = url

    def search_client(
        self, query: SearchClientRequest, timeout=15
    ) -> SDKResponse[SearchClientResponse]:
        query_search = []
        if query.fname:
            query_search.append(f"fname:{query.fname}~")
        if query.lname:
            query_search.append(f"lname:{query.lname}~")
        if query.mname:
            query_search.append(f"mname:{query.mname}~")
        if query.phone:
            query_search.append(f"canonicalphone:{query.phone}~")
        if query.birth:
            query_search.append(f"birth:{query.birth}")
        if query.discountnumber:
            query_search.append(f"discountnumber:{query.discountnumber}~")
        return self._client.get(
            urljoin(str(self._url), "client/_search/"),
            SearchClientResponse,
            data=json.dumps(
                {
                    "query": {
                        "bool": {
                            "must": [
                                {
                                    "query_string": {
                                        "query": " AND ".join(query_search),
                                        "fuzzy_prefix_length": 0,
                                        "fuzziness": "AUTO",
                                        "fuzzy_transpositions": True,
                                        "fuzzy_max_expansions": 100,
                                    }
                                }
                            ]
                        }
                    }
                }
            ),
            timeout=timeout,
            headers={"Content-Type": "application/json"},
        )



from urllib.parse import urljoin

from pydantic import BaseModel, HttpUrl

from ..client import Empty, SDKClient, SDKResponse


class CallBackRequest(BaseModel):
    task_id: str
    data: str
    has_error: bool


class MisSamdService:
    def __init__(self, client: SDKClient):
        self._client = client

    def send_callback(
        self, query: CallBackRequest, url: HttpUrl, timeout=3
    ) -> SDKResponse[Empty]:
        return self._client.post(
            urljoin(str(url), "samd/tasks/callback/"),
            Empty,
            json=query.dict(exclude_none=True),
            timeout=timeout,
        )



from pathlib import Path
from urllib.parse import urljoin

from pydantic import BaseModel, HttpUrl

from .client import SDKClient, SDKResponse


class BarcodeRequest(BaseModel):
    file_path: Path


class BarcodeResponse(BaseModel):
    text: str


class BarcodeRecognizeService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        assert url.endswith("/")
        self._client = client
        self._url = url

    def recognize(
        self, query: BarcodeRequest, timeout=15
    ) -> SDKResponse[BarcodeResponse]:
        files = {"filename": open(str(query.file_path), "rb")}
        return self._client.post(
            urljoin(str(self._url), "upload/"),
            BarcodeResponse,
            files=files,
            timeout=timeout,
        )



import datetime

from typing import List, Optional
from urllib.parse import urljoin

from pydantic import BaseModel, Field, HttpUrl

from .. import Gender
from ..client import SDKClient, SDKResponse
from ..pre_record import OrderResults, Service


MC_URL_BY_ID = {
    1: "https://sm.mm/",
    2: "https://dm.mm/",
    3: "https://tm.mm/",
    4: "https://fm.mm/",
}


class GetClient(BaseModel):
    uuid: str


class SearchClient(BaseModel):
    uuid: Optional[str]
    phone: Optional[str]
    ignore_lab_number: Optional[str] = Field(
        description="Для проверки has_retail можем надо исключать текущий лаб номер если у нас редактирование заявки"
    )
    has_retail: Optional[bool] = Field(
        description="Фильтрация клиентов по наличию у них розничной лаб. заявки"
    )


class Client(BaseModel):
    uuid: str
    phone: str
    last_name: str = Field(alias="lname")
    first_name: str = Field(alias="fname")
    middle_name: Optional[str] = Field(alias="mname", default="")
    birth: datetime.date
    email: Optional[str]
    sex: Gender

    class Config:
        use_enum_values = True


class ClientResultsRequest(BaseModel):
    client_uuid: str
    mc_id: int
    pre_record_number: int
    lab_number: int


class ClientResultsResponse(BaseModel):
    lab_results_url: Optional[str]
    covid_results_url: Optional[str]
    flg_results_url: Optional[str]


class ClientLabOrderResponse(BaseModel):
    number: int
    phone: str
    services: List[Service]
    lname: str
    fname: str
    mname: str
    birth: Optional[datetime.date]
    client_uuid: Optional[str]
    email: Optional[str]
    results: OrderResults
    dc: datetime.datetime


class ClientLabOrderRequest(BaseModel):
    with_pre_record: bool = False
    client_uuid: str


class ClientService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url

    def get_client(self, query: GetClient, timeout=3) -> SDKResponse[Client]:
        return self._client.get(
            urljoin(str(self._url), f"mobil/rest/client/{query.uuid}/"),
            Client,
            timeout=timeout,
        )

    def search(self, query: SearchClient, timeout=3) -> SDKResponse[List[Client]]:
        return self._client.get(
            urljoin(str(self._url), "mobil/rest/client/"),
            Client,
            params=query.dict(exclude_none=True),
            timeout=timeout,
        )

    def get_results(
        self, query: ClientResultsRequest, timeout=3
    ) -> SDKResponse[ClientResultsResponse]:
        params = {
            "lab_number": query.lab_number,
            "pre_record_number": query.pre_record_number,
        }
        return self._client.get(
            urljoin(
                MC_URL_BY_ID[query.mc_id], f"mobil/client/{query.client_uuid}/results/"
            ),
            ClientResultsResponse,
            params=params,
            timeout=timeout,
        )

    def get_lab(
        self, query: ClientLabOrderRequest, timeout=3
    ) -> SDKResponse[List[ClientLabOrderResponse]]:
        return self._client.get(
            urljoin(str(self._url), f"mobil/client/{query.client_uuid}/lab_results/"),
            ClientLabOrderResponse,
            params={"with_pre_records": query.with_pre_record},
            timeout=timeout,
        )



from typing import Any

from pydantic import BaseModel, HttpUrl

from .client import SDKClient, SDKResponse


class SignRequest(BaseModel):
    thumbprint: str
    xml_file: Any


class SignResponse(BaseModel):
    signedContent: str
    filename: str


class CryptoProService:
    def __init__(self, client: SDKClient):
        self._client = client

    def sign(
        self, query: SignRequest, token: str, url: HttpUrl, timeout=3
    ) -> SDKResponse[SignResponse]:
        return self._client.post(
            url + "signer",
            SignResponse,
            params={"thumbprint": query.thumbprint, "api-key": token},
            files={"file": query.xml_file},
            timeout=timeout,
        )



import datetime

from enum import Enum
from typing import List, Optional
from urllib.parse import urljoin

from pydantic import BaseModel, HttpUrl

from .. import Gender
from ..client import SDKClient, SDKResponse


class ConstraintType(str, Enum):
    REQUIRED_FIELDS = "required_fields"
    EXCLUDED_SERVICES = "excluded_services"
    REQUIRED_SERVICES = "required_services"
    PLACE = "place"
    DAY_OF_WEEK = "day_of_week"
    BUSINESS_HOURS_FOR_SERVICE = "business hours for service"


class ConstraintRequiredField(str, Enum):
    DIURESIS_FIELD = "diuresis"
    WEIGHT_FIELD = "weight"
    HEIGHT_FIELD = "height"
    PREGNANCY_WEEK_FIELD = "pregnancy_week"
    CYCLE_DAY_FIELD = "cycle_day"
    CYCLE_PHASE_FIELD = "cycle_phase"
    POST_FIELD = "post"
    WORK_PLACE_PHONE_FIELD = "work_place_phone"
    WORK_PLACE_FIELD = "work_place"
    TRIP_FIELD = "trip"
    WORK_PLACE_ADDRESS_FIELD = "work_place_address"
    TRIP_DATE_FIELD = "trip_date"
    REGISTRATION_CITY = "registration_city"
    REGISTRATION_STREET = "registration_street"
    REGISTRATION_BUILDING = "registration_building"
    REGISTRATION_QUARTER = "registration_quarter"
    PASSPORT_DATE = "passport_date"
    PASSPORT_DEPARTMENT_CODE = "passport_department_code"
    CONTINGENT_COVID = "contingent_covid"


class ConstraintsRequest(BaseModel):
    price_code: Optional[List[str]]
    type: Optional[ConstraintType]
    service_id: Optional[List[str]]

    class Config:
        use_enum_values = True


class GetCostRequest(BaseModel):
    lab_order_id: Optional[int]
    lab_service_ids: Optional[List[int]]
    price_codes: List[str]
    lab_price_id: Optional[int]


class ConstraintsResponse(BaseModel):
    id: int
    price_code: str
    type: ConstraintType
    gender: Optional[Gender]
    med_centers: Optional[List[Optional[str]]]
    excluded_days: Optional[List[Optional[int]]]
    excluded_services: Optional[List[Optional[str]]]
    required_fields: Optional[List[Optional[ConstraintRequiredField]]]
    service_start_time_in_weekdays: Optional[datetime.time]
    service_end_time_in_weekdays: Optional[datetime.time]
    service_start_time_in_weekends: Optional[datetime.time]
    service_end_time_in_weekends: Optional[datetime.time]


class PublicCostResponse(BaseModel):
    price_code: str
    cost: Optional[int]
    discount_cost: Optional[int]


class LabConstraintService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url

    def get_constraints(
        self, query: ConstraintsRequest, timeout=3
    ) -> SDKResponse[List[ConstraintsResponse]]:
        return self._client.get(
            urljoin(str(self._url), "lab/rest/constraints/"),
            ConstraintsResponse,
            params=query.dict(exclude_unset=True),
            timeout=timeout,
        )


class RetailService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url

    def get_public_costs(self, query: GetCostRequest = None, timeout=3) -> SDKResponse[List[PublicCostResponse]]:
        return self._client.get(
            urljoin(str(self._url), "lab/price/external/costs/"),
            PublicCostResponse,
            params=query.dict(exclude_unset=True) if query else None,
            timeout=timeout,
        )



from pydantic import BaseModel

from ..client import SDKClient, SDKResponse


class LocalChangeResponse(BaseModel):
    have_local_changes: bool


class ImportRequest(BaseModel):
    name: str
    import_type: str
    path_to_dump: str
    webdav_ip_address: str
    task_id: int


class ImportResponse(BaseModel):
    success: str


class ExportRequest(BaseModel):
    name: str
    export_type: str
    task_id: int


class ExportResponse(BaseModel):
    success: str


class GetLatestDumpRequest(BaseModel):
    config: str


class GetLatestDumpResponse(BaseModel):
    zip_path: str


class MisStandAloneService:
    def __init__(self, client: SDKClient):
        self._client = client

    def have_changes(self, url, timeout=3) -> SDKResponse[LocalChangeResponse]:
        return self._client.get(url, LocalChangeResponse, timeout=timeout,)

    def import_dump(
        self, url, query: ImportRequest, timeout=5
    ) -> SDKResponse[ImportResponse]:
        return self._client.post(
            url, ImportResponse, data=query.dict(), timeout=timeout,
        )

    def export_dump(
        self, url, query: ExportRequest, timeout=5
    ) -> SDKResponse[ExportResponse]:
        return self._client.post(
            url, ExportResponse, data=query.dict(), timeout=timeout,
        )



import datetime
from typing import Optional, List
from urllib.parse import urljoin

from pydantic import BaseModel, HttpUrl

from ..client import SDKClient, SDKResponse


class UserRequest(BaseModel):
    username: str


class OrgRequest(BaseModel):
    ids: Optional[List[int]]
    name: Optional[str]


class OrgResponse(BaseModel):
    id: int
    name: str
    base_org_id: Optional[int]


class OrgResponsePaginated(BaseModel):
    results: List[OrgResponse]
    count: int
    next: Optional[str]
    previous: Optional[str]
    page_size: int


class MobilOrgService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url

    def get_orgs_by_user(
        self, query: UserRequest, timeout=3
    ) -> SDKResponse[OrgResponse]:
        return self._client.get(
            urljoin(str(self._url), f"mobil/api/org_by_user/"),
            OrgResponse,
            params=query.dict(),
            timeout=timeout,
        )

    def get_orgs(self, query: OrgRequest, timeout=3) -> SDKResponse[OrgResponsePaginated]:
        return self._client.get(
            f"{self._url}/mobil/rest/external_orgs/",
            OrgResponsePaginated,
            timeout=timeout,
            params=query.dict(exclude_unset=True),
            headers={"Content-Type": "application/json"},
        )


class GetManagerRequest(UserRequest):
    name: str


class GetManagerResponse(BaseModel):
    username: str
    name: str
    email: Optional[str]
    groups: List[str] = []
    is_active: bool


class ManagerService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url

    def get_superuser_managers(
        self, query: GetManagerRequest, timeout=3
    ) -> SDKResponse[GetManagerResponse]:
        return self._client.get(
            urljoin(str(self._url), f"mobil/api/user/search/"),
            GetManagerResponse,
            params=query.dict(),
            timeout=timeout,
        )


class ClientMedicineRequest(BaseModel):
    external_id: Optional[str]
    snils: Optional[str]


class Medicine(BaseModel):
    name: str
    date: datetime.date
    expired_date: Optional[datetime.date]


class ProfInfo(BaseModel):
    number: int
    type: str
    result: str
    prp: bool


class CertificateInfo(BaseModel):
    number: int
    conclusion: bool


class ClientMedicineResponse(BaseModel):
    medicine: Optional[List[Medicine]] = []
    certificate: Optional[CertificateInfo] = None
    prof: Optional[ProfInfo] = None


class ClientMedicineService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url

    def medicine_info(
        self, query: ClientMedicineRequest, timeout=15
    ) -> SDKResponse[ClientMedicineResponse]:
        return self._client.get(
            urljoin(str(self._url), f"mobil/api/client/medicine/"),
            ClientMedicineResponse,
            params=query.dict(),
            timeout=timeout,
        )


class DeliveryOrderRequest(BaseModel):
    on_delivery: bool
    date_from: datetime.datetime
    date_to: datetime.datetime
    orgs: List[str]
    last_name: Optional[str]
    first_name: Optional[str]
    middle_name: Optional[str]
    out_isnull: Optional[bool]
    delivery_status: Optional[str]
    manager_ids: Optional[List[str]]
    page: Optional[int]
    page_size: Optional[int]


class DeliveryDetail(BaseModel):
    way: str
    delivery_wait: Optional[str]
    delivery_date: Optional[str]
    address: Optional[str]
    itinerary: Optional[str]


class MedCenterInfo(BaseModel):
    name: str
    address: str
    metro: str


class DeliveryOrder(BaseModel):
    lmk: Optional[DeliveryDetail] = None
    certificate: Optional[DeliveryDetail] = None
    prof: Optional[DeliveryDetail] = None
    client_fio: str
    client_birth: str
    date: str
    med_center: MedCenterInfo
    org_name: str
    org_id: int
    lab_number: Optional[int]


class DeliveryOrderResponse(BaseModel):
    results: Optional[List[DeliveryOrder]] = []
    count: int
    next: Optional[str]
    previous: Optional[str]
    page_size: int


class DeliveryOrderService:
    def __init__(self, client: SDKClient, url: HttpUrl):
        self._client = client
        self._url = url

    def delivery_info(
        self, query: DeliveryOrderRequest, timeout=15
    ) -> SDKResponse[DeliveryOrderResponse]:
        return self._client.get(
            urljoin(str(self._url), f"mobil/api/order/delivery/"),
            DeliveryOrderResponse,
            params=query.dict(),
            timeout=timeout,
        )
